@@ -902,6 +902,8 @@ foldMoveTo(
 	// that moves the cursor is used.
 	lnum_off = 0;
 	gap = &curwin->w_folds;
+	if (gap->ga_len == 0)
+	    break;
 	use_level = FALSE;
 	maybe_small = FALSE;
 	lnum_found = curwin->w_cursor.lnum;
@@ -2422,8 +2424,8 @@ foldUpdateIEMSRecurse(
 							      && flp->lvl > 0)
     {
 	(void)foldFind(gap, startlnum - 1, &fp);
-	if (fp >= ((fold_T *)gap->ga_data) + gap->ga_len
-						   || fp->fd_top >= startlnum)
+	if (fp != NULL && (fp >= ((fold_T *)gap->ga_data) + gap->ga_len
+						   || fp->fd_top >= startlnum))
 	    fp = NULL;
     }
 
