@@ -916,9 +916,13 @@ gui_mch_add_menu(vimmenu_T *menu, int idx)
 # endif
 	{
 	    if (gui.menu_bg_pixel != INVALCOLOR)
+	    {
 		XtSetArg(arg[0], XmNbackground, gui.menu_bg_pixel); n++;
+	    }
 	    if (gui.menu_fg_pixel != INVALCOLOR)
+	    {
 		XtSetArg(arg[1], XmNforeground, gui.menu_fg_pixel); n++;
+	    }
 	    menu->submenu_id = XmCreatePopupMenu(textArea, "contextMenu",
 								      arg, n);
 	    menu->id = (Widget)0;
@@ -1238,11 +1242,10 @@ add_pixmap_args(vimmenu_T *menu, Arg *args, int n)
     }
     else
     {
-# if 0  // DISABLED - this causes a crash when running "make test_gui" in
-	// Test_colorscheme()
 	if (menu->xpm_fname != NULL)
+	{
 	    XtSetArg(args[n], XmNpixmapFile, menu->xpm_fname); n++;
-# endif
+	}
 	XtSetArg(args[n], XmNpixmapData, menu->xpm); n++;
 	XtSetArg(args[n], XmNlabelLocation, XmBOTTOM); n++;
     }
