@@ -196,6 +196,7 @@ update_topline(void)
     // the cursor line.
     if (!screen_valid(TRUE) || curwin->w_height == 0)
     {
+	check_cursor_lnum();
 	curwin->w_topline = curwin->w_cursor.lnum;
 	curwin->w_botline = curwin->w_topline;
 	curwin->w_valid |= VALID_BOTLINE|VALID_BOTLINE_AP;
@@ -2144,6 +2145,10 @@ scroll_cursor_halfway(int atend)
     linenr_T	old_topline = curwin->w_topline;
 #endif
 
+#ifdef FEAT_PROP_POPUP
+    // if the width changed this needs to be updated first
+    may_update_popup_position();
+#endif
     loff.lnum = boff.lnum = curwin->w_cursor.lnum;
 #ifdef FEAT_FOLDING
     (void)hasFolding(loff.lnum, &loff.lnum, &boff.lnum);
